/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.DataTypes;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.Vector;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;

public class DataCollection {

	private ProbeSet probeSet = new ProbeSet();
	private Vector dataSets = new Vector();
	private Vector dataGroups = new Vector();
	private DataStore activeDataStore = null;
	private Vector listeners = new Vector();
	
	public DataSet [] getAllDataSets () {
		DataSet [] sets = (DataSet [])dataSets.toArray(new DataSet[0]);
		Arrays.sort(sets);
		return sets;
	}
	
	public DataSet getDataSet (String name) {
		Enumeration e = dataSets.elements();
		while (e.hasMoreElements()) {
			DataSet s = (DataSet)e.nextElement();
			if (s.name().equals(name)) {
				return s;
			}
		}
		return null;
	}
	
	public DataGroup getDataGroup (String name) {
		Enumeration e = dataGroups.elements();
		while (e.hasMoreElements()) {
			DataGroup g = (DataGroup)e.nextElement();
			if (g.name().equals(name)) {
				return g;
			}
		}
		return null;
	}
		
	public ProbeSet probeSet () {
		return probeSet;
	}
	
	public void addDataSet (DataSet data) {
		dataSets.add(data);
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((DataChangeListener)e.nextElement()).dataSetAdded(data);
		}
	}
	
	public void addDataGroup (DataGroup group) {
		dataGroups.add(group);
		group.setCollection(this);
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((DataChangeListener)e.nextElement()).dataGroupAdded(group);
		}
	}
	
	public void removeDataGroup (DataGroup group) {
		dataGroups.remove(group);
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((DataChangeListener)e.nextElement()).dataGroupRemoved(group);
		}

	}
	
	public void removeDataSet (DataSet data) {
		dataSets.remove(data);
		Enumeration e = dataGroups.elements();
		while (e.hasMoreElements()) {
			DataGroup g = (DataGroup)e.nextElement();

			if (g.containsDataSet(data)) {
				g.removeDataSet(data);
			}
		
		}
		e = listeners.elements();
		while (e.hasMoreElements()) {
			((DataChangeListener)e.nextElement()).dataSetRemoved(data);
		}

	}
	
	public DataGroup [] getAllDataGroups () {
		DataGroup [] groups = (DataGroup [])dataGroups.toArray(new DataGroup[0]);
		Arrays.sort(groups);
		return groups;
	}
	
	public void setActiveDataStore (DataStore d) throws ChipMonkException {
		if (d == null || dataSets.contains(d) || dataGroups.contains(d)) {
			activeDataStore = d;
		}
		else {
			throw new ChipMonkException("Data store "+d.name()+" could not be found in the data collection");
		}
	}
	
	public DataStore getActiveDataStore () {
		return activeDataStore;
	}
	
	public void addDataChangeListener (DataChangeListener l) {
		listeners.add(l);
	}
	
	public void removeDataChangeListener (DataChangeListener l) {
		listeners.remove(l);
	}
	
	public void dataGroupRenamed (DataGroup g) {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((DataChangeListener)e.nextElement()).dataGroupRenamed(g);
		}
	}
	
	public void dataSetRenamed (DataSet s) {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((DataChangeListener)e.nextElement()).dataSetRenamed(s);
		}
	}
	
	public void dataGroupSamplesChanged (DataGroup g) {
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((DataChangeListener)e.nextElement()).dataGroupSamplesChanged(g);
		}
	}
}
