/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Chromosome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Feature;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Location;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;


public class FeatureFilter extends JDialog implements ActionListener, KeyListener, MouseListener, Runnable, ItemListener {

	private ChipMonkApplication application;
	private JComboBox annotationPosition;
	private JComboBox annotationType;
	private JTextField annotationLimit;
	private JButton okButton;
	private FilterProgressDialog progressDialog;

	public FeatureFilter (ChipMonkApplication application) {
		super(application,"Feature Filter...");
		setSize(500,150);
		setLocationRelativeTo(application);
		
		this.application = application;

		getContentPane().setLayout(new BorderLayout());
		
		JPanel choicePanel = new JPanel();
		choicePanel.setLayout(new BoxLayout(choicePanel,BoxLayout.Y_AXIS));
		
		JPanel choicePanel0 = new JPanel();
		JLabel probeListLabel = new JLabel ("Starting with probes in '"+application.dataCollection().probeSet().getActiveProbeListName()+"' ("+application.dataCollection().probeSet().getActiveProbes().length+" probes)",JLabel.CENTER);
		probeListLabel.setFont(new Font("Default",Font.BOLD,12));
		choicePanel0.add(probeListLabel);
		choicePanel.add(choicePanel0);
		
		JPanel choicePanel1 = new JPanel();
		
		choicePanel1.add(new JLabel("Select probes which are "));
		annotationPosition = new JComboBox(new String [] {"surrounded by", "close to", "upstream of", "downstream of", "surrounded by or upstream of", "surrounded by or downstream of"});
		annotationPosition.addItemListener(this);
		choicePanel1.add(annotationPosition);
		annotationType = new JComboBox(application.genome().listAvailableFeatureTypes());
		choicePanel1.add(annotationType);
		choicePanel.add(choicePanel1);
		
		
		JPanel choicePanel2 = new JPanel();
		choicePanel2.add(new JLabel("Feature distance cutoff "));
		annotationLimit = new JTextField("2000",7);
		annotationLimit.addKeyListener(this);
		annotationLimit.setEnabled(false);
		choicePanel2.add(annotationLimit);
		choicePanel2.add(new JLabel("bp"));
		choicePanel.add(choicePanel2);
				
		getContentPane().add(choicePanel,BorderLayout.CENTER);

		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		okButton = new JButton("Run Filter");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		buttonPanel.add(okButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setVisible(true);
	}

	private void filterProbes () {
		
		String annotationTypeValue = (String)annotationType.getSelectedItem();
		int distanceLimit = 0;
				
		String annotationPositionValue = (String)annotationPosition.getSelectedItem();
		// We're going to set up a set of booleans which tell us which kinds
		// of relationships we're allowed to look for later.
		boolean surrounding = true;
		boolean upstream = true;
		boolean downstream = true;
		
		if (annotationPositionValue.equals("surrounded by")) {
			upstream = false;
			downstream = false;
		}
		else if (annotationPositionValue.equals("close to")) {
			// Keep all default options
		}
		else if (annotationPositionValue.equals("surrounded by or upstream of")) {
			downstream = false;
		}
		else if (annotationPositionValue.equals("surrounded by or downstream of")) {
			upstream = false;
		}
		else if (annotationPositionValue.equals("upstream of")) {
			surrounding = false;
			downstream = false;
		}
		else if (annotationPositionValue.equals("downstream of")) {
			surrounding = false;
			upstream = false;
		}
		else {
			System.err.println("Didn't recognise position value '"+annotationPositionValue+"'");
			return;
		}
	
		// We only need to worry about a distance cutoff if we're not using
		// surrounding.
		if (!annotationTypeValue.equals("surrounding")) {
			if (annotationLimit.getText().length()> 0) {
				distanceLimit = Integer.parseInt(annotationLimit.getText());
			}
		}
				
		ProbeList passedProbes = new ProbeList(application.dataCollection().probeSet(),"","No value");
		
		// Since we're going to be making the annotations on the
		// basis of position we should go through all probes one
		// chromosome at a time.
		
		Chromosome [] chrs = application.genome().getAllChromosomes();
		
		for (int c=0;c<chrs.length;c++) {
			
			progressDialog.updateProgress(c, chrs.length);
			try {
				Thread.sleep(10);
			} 
			catch (InterruptedException e1) {

			}
			
			Probe [] probes = application.dataCollection().probeSet().getActiveProbesForChromosome(chrs[c]);
			Feature [] features = chrs[c].getFeaturesForType(annotationTypeValue);
			
			// We can now step through the probes looking for the best feature match
			for (int p=0;p<probes.length;p++) {
				
				for (int f=0;f<features.length;f++) {
					
					if (surrounding) {
						// Quickest check is whether a probe is inside a feature
					
						if (probes[p].position() > features[f].location().start() && probes[p].position() < features[f].location().end()) {
							passedProbes.addProbe(probes[p],null);
							break;
						}
					}
					
					if (upstream) {
						// Check if we're upstream
						// Get the distance to the start
						int d=0;
						if (features[f].location().strand() == Location.FORWARD) {
							d = features[f].location().start() - probes[p].position();
						}
						else {
							d = probes[p].position() - features[f].location().end();
						}
						
						if (d >= 0) {
							// We're the right side of the feature

							if (d < distanceLimit) {
								passedProbes.addProbe(probes[p],null);
								break;
							}
						}
					}

					if (downstream) {
						// Check if we're downstream
						// Get the distance to the start
						int d=0;
						if (features[f].location().strand() == Location.FORWARD) {
							d = probes[p].position() - features[f].location().end();
						}
						else {
							d = features[f].location().start() - probes[p].position();
						}
						
						if (d >= 0) {
							// We're the right side of the feature

							if (d < distanceLimit) {
								passedProbes.addProbe(probes[p],null);
								break;
							}
						}
					}
				}
			}
		}
		
		progressDialog.processingFinished();
		
		// See if any probes actually passed
		if (passedProbes.getAllProbes().length == 0) {
			JOptionPane.showMessageDialog(this,"No probes matched the criteria set","Info",JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		
		// Ask for a name for the list
		String groupName=null;
		while (true) {
			groupName = (String)JOptionPane.showInputDialog(this,"Enter list name","Found "+passedProbes.getAllProbes().length+" probes",JOptionPane.QUESTION_MESSAGE,null,null,"Probes "+annotationPositionValue+" "+annotationTypeValue);
			if (groupName == null)
				return;  // They cancelled
				
				
			if (groupName.length() == 0)
				continue; // Try again
			
			break;
		}

		passedProbes.setName(groupName);
		
		try {
			application.dataCollection().probeSet().addProbeList(passedProbes);
		} catch (ChipMonkException e) {
			e.printStackTrace();
		}
	}
	
	private void checkOK () {
		// Check to see if enough information has been added to allow us to
		// enable the OK button.
		
		// If we get here then we're good to go
		okButton.setEnabled(true);
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("cancel")) {
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("ok")) {
			progressDialog = new FilterProgressDialog(this);
			Thread t = new Thread(this);
			t.start();
		}		
	}

	public void keyTyped(KeyEvent arg0) {
	}

	public void keyPressed(KeyEvent ke) {

	}

	public void keyReleased(KeyEvent ke) {
		JTextField f = (JTextField)ke.getSource();
		
		char [] c = f.getText().toCharArray();
		
		StringBuffer b = new StringBuffer();
		for (int i=0;i<c.length;i++) {
			
			if (Character.isDigit(c[i])) {
				b.append(c[i]);
				continue;
			}

			f.setText(b.toString());
			break;
		}
		
		
		checkOK();
	}
		

	public void mouseClicked(MouseEvent me) {
		//We're only interested in double clicks
		if (me.getClickCount() !=2 ) return;
		// This is only linked from the report JTable
		JTable t = (JTable)me.getSource();
		int r = t.getSelectedRow();
		Probe p = (Probe)t.getValueAt(r,0);
		application.displayChromosome(p.chromosome(),p.position());
		application.chromosomeViewer().setView(p.position()-100,p.position()+100);
	}

	public void mousePressed(MouseEvent arg0) {
	}
	public void mouseReleased(MouseEvent arg0) {
	}
	public void mouseEntered(MouseEvent arg0) {
	}
	public void mouseExited(MouseEvent arg0) {
	}

	public void run() {
		filterProbes();
	}

	public void itemStateChanged(ItemEvent arg0) {
		if (annotationPosition.getSelectedItem().equals("surrounded by")) {
			annotationLimit.setEnabled(false);
		}
		else {
			annotationLimit.setEnabled(true);
		}
	}
	
}
