/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics.AnovaTest;
import uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics.BenjHochFDR;
import uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics.OneSampleTTest;
import uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics.ProbeGroupTTestValue;
import uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics.TwoSampleTTest;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Chromosome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataGroup;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Renderers.TypeColourRenderer;


public class WindowedReplicateStatsFilter extends JDialog implements ActionListener, KeyListener, ListSelectionListener, Runnable {

	private ChipMonkApplication application;
	private JList dataList;
	private JTextField cutoff;
	private JTextField windowSize;
	private JCheckBox multiTest;
	private JButton okButton;
	private final double log2 = Math.log(2);
	private FilterProgressDialog progressDialog;
	private String analysisType;

	public WindowedReplicateStatsFilter (ChipMonkApplication application) {
		super(application,"Windowed Replicate T-test filter...");
		setSize(500,300);
		setLocationRelativeTo(application);
		
		this.application = application;

		getContentPane().setLayout(new BorderLayout());
		JPanel dataPanel = new JPanel();
		dataPanel.setLayout(new BorderLayout());
		dataPanel.setBorder(BorderFactory.createEmptyBorder(4,4,0,4));

		dataPanel.add(new JLabel("Data Store(s) to Test",JLabel.CENTER),BorderLayout.NORTH);
		
		DefaultListModel dataModel = new DefaultListModel();
		
		DataSet [] sets = application.dataCollection().getAllDataSets();
		for (int i=0;i<sets.length;i++) {
			dataModel.addElement(sets[i]);
		}
		DataGroup [] groups = application.dataCollection().getAllDataGroups();
		for (int i=0;i<groups.length;i++) {
				dataModel.addElement(groups[i]);
		}
		
		
		dataList = new JList(dataModel);
		dataList.setCellRenderer(new TypeColourRenderer());
		dataList.addListSelectionListener(this);
		dataPanel.add(new JScrollPane(dataList),BorderLayout.CENTER);
		
		getContentPane().add(dataPanel,BorderLayout.WEST);
		
		JPanel choicePanel = new JPanel();

		choicePanel.setLayout(new GridBagLayout());
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.weightx=0.2;
		gbc.weighty=0.5;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		
		gbc.gridwidth=2;
		JLabel probeListLabel = new JLabel ("Testing probes in '"+application.dataCollection().probeSet().getActiveProbeListName()+"' ("+application.dataCollection().probeSet().getActiveProbes().length+" probes)",JLabel.CENTER);
		probeListLabel.setFont(new Font("Default",Font.BOLD,12));
		choicePanel.add(probeListLabel,gbc);
		
		gbc.gridwidth=1;
		gbc.gridy++;
		
		choicePanel.add(new JLabel("P-value cutoff",JLabel.RIGHT),gbc);
	
		gbc.gridx++;
		gbc.weightx=0.6;
		
		cutoff = new JTextField("0.01",5);
		cutoff.addKeyListener(this);
		choicePanel.add(cutoff,gbc);

		gbc.gridx=0;
		gbc.gridy++;
		gbc.weightx=0.2;
		
		choicePanel.add(new JLabel("Window size (bp)",JLabel.RIGHT),gbc);
		
		gbc.gridx++;
		gbc.weightx=0.6;
		
		windowSize = new JTextField("1000",5);
		windowSize.addKeyListener(this);
		choicePanel.add(windowSize,gbc);
		
		gbc.gridx=0;
		gbc.gridy++;
		gbc.weightx=0.2;
		
		choicePanel.add(new JLabel("Apply multiple testing correction",JLabel.RIGHT),gbc);
		
		gbc.gridx++;
		gbc.weightx=0.6;

		
		multiTest = new JCheckBox();
		multiTest.setSelected(true);
//		multiTest.setEnabled(false);
		choicePanel.add(multiTest,gbc);
		
		getContentPane().add(new JScrollPane(choicePanel),BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		okButton = new JButton("Run Filter");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		okButton.setEnabled(false);
		buttonPanel.add(okButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setVisible(true);
	}

	private void makeSingleProbeList () {
		
		DataStore g = (DataStore)dataList.getSelectedValue();
		
		Chromosome [] chromosomes = application.genome().getAllChromosomes();
		
		double cutoffValue = Double.parseDouble(cutoff.getText());
		int windowSizeValue = Integer.parseInt(windowSize.getText());
		Vector newListProbesVector = new Vector();
		
		for (int c=0;c<chromosomes.length;c++) {
			
			progressDialog.updateProgress(c, chromosomes.length);
			try {
				Thread.sleep(10);
			} 
			catch (InterruptedException e1) {

			}
				
			Probe [] probes = application.dataCollection().probeSet().getActiveProbesForChromosome(chromosomes[c]);
			int startIndex = 0;
			int endIndex = -1;
			
			while (true) {
				
				// Create the next set of probes.
				// We always start by adding 1 to the end index
				endIndex ++;
				
				// Bail out if we're at the end
				if (endIndex == probes.length) break;
				
				// Now remove probes from the front until we're under
				// the cutoff length
				while ((probes[endIndex].position() - probes[startIndex].position()) >windowSizeValue) {
					++startIndex;
				}
				
				// Now add probes to the end again until just before we hit
				// the window size
				while (endIndex+1 < probes.length && (probes[endIndex+1].position() - probes[startIndex].position())<windowSizeValue) {
					++endIndex;
				}
				
//				System.out.println("Looking at chr "+chromosomes[c].name()+" probes "+startIndex+"-"+endIndex);
				// We now have a set of probes we're going to use for the test.
				Probe [] theseProbes = new Probe[(endIndex-startIndex)+1];
				for (int j=startIndex;j<=endIndex;j++){
					theseProbes[j-startIndex] = probes[j];
				}
		
				// We need at least 3 probes in a set to calcualte a p-value
				if (theseProbes.length < 3) continue;
				
				
				double [] values = new double [theseProbes.length];
				for (int i=0;i<theseProbes.length;i++) {
					values[i] = Math.log(g.getRatioForProbe(theseProbes[i]))/log2;
				}
				
//				StringBuffer sb = new StringBuffer();
//				sb.append("Values for probes ");
//				sb.append(startIndex);
//				sb.append(" to ");
//				sb.append(endIndex);
//				sb.append(" are");
//				for (int p=0;p<values.length;p++) {
//					sb.append(":");
//					sb.append(values[p]);
//				}
//				
//				System.out.println(sb.toString());
				
				double pValue = 0;
				try {
					pValue = OneSampleTTest.calculatePValue(values,0);
				} catch (ChipMonkException e) {
					progressDialog.exceptionReceived(e);
					e.printStackTrace();
				}
//				System.out.println("P-value is "+pValue);
				ProbeGroupTTestValue v = new ProbeGroupTTestValue(theseProbes,pValue);
				newListProbesVector.add(v);
			}
		}
		
		ProbeGroupTTestValue [] newListProbes = (ProbeGroupTTestValue []) newListProbesVector.toArray(new ProbeGroupTTestValue[0]);
		
		
		// If multi-testing correction is needed then do it...
		if (multiTest.isSelected()) {
			BenjHochFDR.calculateQValues(newListProbes);
		}
		
		ProbeList newList;
		
		if (multiTest.isSelected()) {
			newList = new ProbeList(application.dataCollection().probeSet(),"","Q-value");
			for (int i=0;i<newListProbes.length;i++) {
				if (newListProbes[i].q <= cutoffValue) {
					Probe [] passedProbes = newListProbes[i].probes;
					for (int p=0;p<passedProbes.length;p++) {
						if (newList.containsProbe(passedProbes[p])) {
							// We always give a probe the lowest possible q-value
							if (newList.getValueForProbe(passedProbes[p]).doubleValue()>newListProbes[i].q) {
								newList.addProbe(passedProbes[p],new Double(newListProbes[i].q));								
							}
						}
						else {
							newList.addProbe(passedProbes[p],new Double(newListProbes[i].q));
						}
					}
				}
			}
		}
		else {
			newList = new ProbeList(application.dataCollection().probeSet(),"","P-value");
			for (int i=0;i<newListProbes.length;i++) {
				if (newListProbes[i].p <= cutoffValue) {
					Probe [] passedProbes = newListProbes[i].probes;
					for (int p=0;p<passedProbes.length;p++) {
						if (newList.containsProbe(passedProbes[p])) {
							// We always give a probe the lowest possible q-value
							if (newList.getValueForProbe(passedProbes[p]).doubleValue()>newListProbes[i].p) {
								newList.addProbe(passedProbes[p],new Double(newListProbes[i].p));								
							}
						}
						else {
							newList.addProbe(passedProbes[p],new Double(newListProbes[i].p));
						}
					}
				}
			}
		}
		progressDialog.processingFinished();

		// See if any probes actually passed
		if (newList.getAllProbes().length == 0) {
			JOptionPane.showMessageDialog(this,"No probes matched the criteria set","Info",JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		
		// Ask for a name for the list
		String groupName=null;
		while (true) {
			groupName = (String)JOptionPane.showInputDialog(this,"Enter list name","Found "+newList.getAllProbes().length+" probes",JOptionPane.QUESTION_MESSAGE,null,null,"Single rep t-test e<"+cutoffValue+" "+windowSizeValue+"bp win");
			if (groupName == null)
				return;  // They cancelled
				
				
			if (groupName.length() == 0)
				continue; // Try again
			
			break;
		}

		newList.setName(groupName);
		
		try {
			application.dataCollection().probeSet().addProbeList(newList);
		} catch (ChipMonkException e) {
			e.printStackTrace();
		}
		
//		setVisible(false);
//		dispose();
		
	}
	
	private void makeDoubleProbeList () {
		
		Object [] o = dataList.getSelectedValues();
		
		DataStore [] g = new DataStore [2];
		g[0] = (DataStore)o[0];
		g[1] = (DataStore)o[1];
		
		Chromosome [] chromosomes = application.genome().getAllChromosomes();
		
		double cutoffValue = Double.parseDouble(cutoff.getText());
		int windowSizeValue = Integer.parseInt(windowSize.getText());
		Vector newListProbesVector = new Vector();
		
		for (int c=0;c<chromosomes.length;c++) {
				
			progressDialog.updateProgress(c, chromosomes.length);
			try {
				Thread.sleep(10);
			} 
			catch (InterruptedException e1) {

			}

			Probe [] probes = application.dataCollection().probeSet().getActiveProbesForChromosome(chromosomes[c]);
			int startIndex = 0;
			int endIndex = -1;
			
			while (true) {
				
				// Create the next set of probes.
				// We always start by adding 1 to the end index
				endIndex ++;
				
				// Bail out if we're at the end
				if (endIndex == probes.length) break;
				
				// Now remove probes from the front until we're under
				// the cutoff length
				while ((probes[endIndex].position() - probes[startIndex].position()) >windowSizeValue) {
					++startIndex;
				}
				
				// Now add probes to the end again until just before we hit
				// the window size
				while (endIndex+1 < probes.length && (probes[endIndex+1].position() - probes[startIndex].position())<windowSizeValue) {
					++endIndex;
				}
				
//				System.out.println("Looking at chr "+chromosomes[c].name()+" probes "+startIndex+"-"+endIndex);

				// We now have a set of probes we're going to use for the test.
				Probe [] theseProbes = new Probe[(endIndex-startIndex)+1];
				for (int j=startIndex;j<=endIndex;j++){
					theseProbes[j-startIndex] = probes[j];
				}

				// We need at least 3 probes in a set to calcualte a p-value
				if (theseProbes.length < 3) continue;

				
				double [] firstValues = new double [theseProbes.length];
				for (int i=0;i<theseProbes.length;i++) {
					firstValues[i] = Math.log(g[0].getRatioForProbe(theseProbes[i]))/log2;
				}
			
				double [] secondValues = new double [theseProbes.length];
				for (int i=0;i<theseProbes.length;i++) {
					secondValues[i] = Math.log(g[1].getRatioForProbe(theseProbes[i]))/log2;
				}
						
				double pValue = 0;
				try {
					pValue = TwoSampleTTest.calculatePValue(firstValues,secondValues);
				} catch (ChipMonkException e) {
					e.printStackTrace();
				}
				
//				System.out.println("First values are:");
//				for (int i=0;i<firstValues.length;i++) {
//					System.out.println(firstValues[i]);
//				}
//				System.out.println("Second values are:");
//				for (int i=0;i<firstValues.length;i++) {
//					System.out.println(secondValues[i]);
//				}
//				System.out.println("Uncorrected p-value is "+pValue);

			
				newListProbesVector.add(new ProbeGroupTTestValue(theseProbes,pValue));
			}
		}

		ProbeGroupTTestValue [] newListProbes = (ProbeGroupTTestValue [])newListProbesVector.toArray(new ProbeGroupTTestValue[0]);
		
		// Do the multi-testing correction if necessary
		if (multiTest.isSelected()) {
			BenjHochFDR.calculateQValues(newListProbes);
		}
		
		ProbeList newList;
		
		if (multiTest.isSelected()) {
			newList = new ProbeList(application.dataCollection().probeSet(),"","Q-value");
			for (int i=0;i<newListProbes.length;i++) {
				if (newListProbes[i].q <= cutoffValue) {
					Probe [] passedProbes = newListProbes[i].probes;
					for (int p=0;p<passedProbes.length;p++) {
						if (newList.containsProbe(passedProbes[p])) {
							// We always give a probe the lowest possible q-value
							if (newList.getValueForProbe(passedProbes[p]).doubleValue()>newListProbes[i].q) {
								newList.addProbe(passedProbes[p],new Double(newListProbes[i].q));								
							}
						}
						else {
							newList.addProbe(passedProbes[p],new Double(newListProbes[i].q));
						}
					}
				}
			}
		}
		else {
			newList = new ProbeList(application.dataCollection().probeSet(),"","P-value");
			for (int i=0;i<newListProbes.length;i++) {
				if (newListProbes[i].p <= cutoffValue) {
					Probe [] passedProbes = newListProbes[i].probes;
					for (int p=0;p<passedProbes.length;p++) {
						if (newList.containsProbe(passedProbes[p])) {
							// We always give a probe the lowest possible q-value
							if (newList.getValueForProbe(passedProbes[p]).doubleValue()>newListProbes[i].p) {
								newList.addProbe(passedProbes[p],new Double(newListProbes[i].p));								
							}
						}
						else {
							newList.addProbe(passedProbes[p],new Double(newListProbes[i].p));
						}
					}
				}
			}
		}

		progressDialog.processingFinished();
		
		// See if any probes actually passed
		if (newList.getAllProbes().length == 0) {
			JOptionPane.showMessageDialog(this,"No probes matched the criteria set","Info",JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		
		// Ask for a name for the list
		String groupName=null;
		while (true) {
			groupName = (String)JOptionPane.showInputDialog(this,"Enter list name","Found "+newList.getAllProbes().length+" probes",JOptionPane.QUESTION_MESSAGE,null,null,"Double rep t-test e<"+cutoffValue+" "+windowSizeValue+"bp win");
			if (groupName == null)
				return;  // They cancelled
				
				
			if (groupName.length() == 0)
				continue; // Try again
			
			break;
		}
		newList.setName(groupName);
		
		
		
		try {
			application.dataCollection().probeSet().addProbeList(newList);
		} catch (ChipMonkException e) {
			e.printStackTrace();
		}
		
//		setVisible(false);
//		dispose();
		
	}
	
	private void makeAnovaProbeList () {
		
		Object [] o = dataList.getSelectedValues();
		
		DataStore [] g = new DataStore [o.length];
		for (int i=0;i<o.length;i++){
			g[i] = (DataStore)o[i];
		}
		
		Chromosome [] chromosomes = application.genome().getAllChromosomes();
		
		double cutoffValue = Double.parseDouble(cutoff.getText());
		int windowSizeValue = Integer.parseInt(windowSize.getText());
		Vector newListProbesVector = new Vector();
		
		for (int c=0;c<chromosomes.length;c++) {
				
			progressDialog.updateProgress(c, chromosomes.length);
			try {
				Thread.sleep(10);
			} 
			catch (InterruptedException e1) {

			}

			Probe [] probes = application.dataCollection().probeSet().getActiveProbesForChromosome(chromosomes[c]);
			int startIndex = 0;
			int endIndex = -1;
			
			while (true) {
				
				// Create the next set of probes.
				// We always start by adding 1 to the end index
				endIndex ++;
				
				// Bail out if we're at the end
				if (endIndex == probes.length) break;
				
				// Now remove probes from the front until we're under
				// the cutoff length
				while ((probes[endIndex].position() - probes[startIndex].position()) >windowSizeValue) {
					++startIndex;
				}
				
				// Now add probes to the end again until just before we hit
				// the window size
				while (endIndex+1 < probes.length && (probes[endIndex+1].position() - probes[startIndex].position())<windowSizeValue) {
					++endIndex;
				}
				
//				System.out.println("Looking at chr "+chromosomes[c].name()+" probes "+startIndex+"-"+endIndex);

				// We now have a set of probes we're going to use for the test.
				Probe [] theseProbes = new Probe[(endIndex-startIndex)+1];
				for (int j=startIndex;j<=endIndex;j++){
					theseProbes[j-startIndex] = probes[j];
				}

				// We need at least 3 probes in a set to calcualte a p-value
				if (theseProbes.length < 3) continue;

				double [][] values = new double[g.length][theseProbes.length];
				for (int i=0;i<theseProbes.length;i++) {
					for (int j=0;j<g.length;j++) {
						values[j][i] = Math.log(g[j].getRatioForProbe(theseProbes[i]))/log2;
					}
				}
									
				double pValue = 0;
				try {
					pValue = AnovaTest.calculatePValue(values);
				} catch (ChipMonkException e) {
					e.printStackTrace();
				}
				
//				System.out.println("First values are:");
//				for (int i=0;i<firstValues.length;i++) {
//					System.out.println(firstValues[i]);
//				}
//				System.out.println("Second values are:");
//				for (int i=0;i<firstValues.length;i++) {
//					System.out.println(secondValues[i]);
//				}
//				System.out.println("Uncorrected p-value is "+pValue);

			
				newListProbesVector.add(new ProbeGroupTTestValue(theseProbes,pValue));
			}
		}

		ProbeGroupTTestValue [] newListProbes = (ProbeGroupTTestValue [])newListProbesVector.toArray(new ProbeGroupTTestValue[0]);
		
		// Do the multi-testing correction if necessary
		if (multiTest.isSelected()) {
			BenjHochFDR.calculateQValues(newListProbes);
		}
		
		ProbeList newList;
		
		if (multiTest.isSelected()) {
			newList = new ProbeList(application.dataCollection().probeSet(),"","Q-value");
			for (int i=0;i<newListProbes.length;i++) {
				if (newListProbes[i].q <= cutoffValue) {
					Probe [] passedProbes = newListProbes[i].probes;
					for (int p=0;p<passedProbes.length;p++) {
						if (newList.containsProbe(passedProbes[p])) {
							// We always give a probe the lowest possible q-value
							if (newList.getValueForProbe(passedProbes[p]).doubleValue()>newListProbes[i].q) {
								newList.addProbe(passedProbes[p],new Double(newListProbes[i].q));								
							}
						}
						else {
							newList.addProbe(passedProbes[p],new Double(newListProbes[i].q));
						}
					}
				}
			}
		}
		else {
			newList = new ProbeList(application.dataCollection().probeSet(),"","P-value");
			for (int i=0;i<newListProbes.length;i++) {
				if (newListProbes[i].p <= cutoffValue) {
					Probe [] passedProbes = newListProbes[i].probes;
					for (int p=0;p<passedProbes.length;p++) {
						if (newList.containsProbe(passedProbes[p])) {
							// We always give a probe the lowest possible q-value
							if (newList.getValueForProbe(passedProbes[p]).doubleValue()>newListProbes[i].p) {
								newList.addProbe(passedProbes[p],new Double(newListProbes[i].p));								
							}
						}
						else {
							newList.addProbe(passedProbes[p],new Double(newListProbes[i].p));
						}
					}
				}
			}
		}

		progressDialog.processingFinished();
		
		// See if any probes actually passed
		if (newList.getAllProbes().length == 0) {
			JOptionPane.showMessageDialog(this,"No probes matched the criteria set","Info",JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		
		// Ask for a name for the list
		String groupName=null;
		while (true) {
			groupName = (String)JOptionPane.showInputDialog(this,"Enter list name","Found "+newList.getAllProbes().length+" probes",JOptionPane.QUESTION_MESSAGE,null,null,"Double rep t-test e<"+cutoffValue+" "+windowSizeValue+"bp win");
			if (groupName == null)
				return;  // They cancelled
				
				
			if (groupName.length() == 0)
				continue; // Try again
			
			break;
		}
		newList.setName(groupName);
		
		
		
		try {
			application.dataCollection().probeSet().addProbeList(newList);
		} catch (ChipMonkException e) {
			e.printStackTrace();
		}
		
//		setVisible(false);
//		dispose();
		
	}
	
	private void checkOK () {
		// Check to see if enough information has been added to allow us to
		// enable the OK button.
		
		// We have to have a dataset selected.
		if (dataList.getSelectedIndices().length == 0) {
			okButton.setEnabled(false);
			return;
		}
		
		// We need a cutoff
		if (cutoff.getText().length()==0) {
			okButton.setEnabled(false);
			return;
		}
		
		// We need a window size
		if (windowSize.getText().length()==0) {
			okButton.setEnabled(false);
			return;
		}
				
		// If we get here then we're good to go
		okButton.setEnabled(true);
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("cancel")) {
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("ok")) {
			if (dataList.getSelectedIndices().length == 1) {
				analysisType = "single";
			}
			else if (dataList.getSelectedIndices().length == 2) {
				analysisType = "double";
			}
			else {
				analysisType = "anova";
			}
			progressDialog = new FilterProgressDialog(this);
			Thread t = new Thread(this);
			t.start();
		}
	}

	public void keyTyped(KeyEvent arg0) {
	}

	public void keyPressed(KeyEvent ke) {

	}

	public void keyReleased(KeyEvent ke) {
		JTextField f = (JTextField)ke.getSource();
		
		char [] c = f.getText().toCharArray();
		
		StringBuffer b = new StringBuffer();
		boolean usedDecimal = false;
		
		// We don't allow decimals in the window length
		if (ke.getSource() == windowSize) {
			usedDecimal = true;
		}
		for (int i=0;i<c.length;i++) {
			
			
			if (Character.isDigit(c[i])) {
				b.append(c[i]);
				continue;
			}
			if (c[i] == KeyEvent.VK_PERIOD) {
				if (usedDecimal)
					continue;
				
				b.append(c[i]);
				usedDecimal = true;
				continue;
			}
			f.setText(b.toString());
			break;
		}
		
		
		checkOK();
	}

	public void valueChanged(ListSelectionEvent lse) {
		checkOK();
	}

	public void run() {
		if (analysisType.equals("single"))
			makeSingleProbeList();
		
		else if (analysisType.equals("double"))
			makeDoubleProbeList();

		else if (analysisType.equals("anova"))
			makeAnovaProbeList();
	}

	
}
