/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;

public class GenomeDownloadSelector extends JDialog {
	
	private ChipMonkApplication application;
	
	private JTree tree;
	private JButton downloadButton;
	
	public GenomeDownloadSelector (ChipMonkApplication application) throws MalformedURLException, IOException, ChipMonkException {
		super(application,"Select Genome to Download...");
		this.application = application;
		setSize(300,250);
		setLocationRelativeTo(application);
		setModal(true);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

		getContentPane().setLayout(new BorderLayout());

		// Create the tree of available genomes
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("Genomes");
		
		URL genomeIndexURL = new URL(ChipMonkPreferences.getInstance().getGenomeDownloadLocation()+"genome_list.txt");

		BufferedReader genomeIndexReader = new BufferedReader(new InputStreamReader(genomeIndexURL.openStream()));
		Hashtable seenSpecies = new Hashtable();
		
		
		String indexLine = null;
		while ((indexLine = genomeIndexReader.readLine())!= null) {
			String [] sections = indexLine.split("\\t");
			if (sections.length != 3) {
				throw new ChipMonkException("Genome list file is corrupt.  Expected 3 sections on line '"+indexLine+"' but got "+sections.length);
			}
			if (seenSpecies.containsKey(new String(sections[0]))) {
				((DefaultMutableTreeNode)seenSpecies.get(sections[0])).add(new GenomeNode(sections[0],sections[1],Integer.parseInt(sections[2])));
			}
			else {
				DefaultMutableTreeNode species = new DefaultMutableTreeNode(new String(sections[0]));
				seenSpecies.put(new String(sections[0]),species);
				root.add(species);
				species.add(new GenomeNode(sections[0],sections[1],Integer.parseInt(sections[2])));
			}
//			System.out.println("Found organism "+sections[0]+" and assembly "+sections[1]);
		}
		
		DefaultTreeModel treeModel = new DefaultTreeModel(root);
		tree = new JTree(treeModel);
		tree.addTreeSelectionListener(new TreeListener());
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);


		getContentPane().add(new JScrollPane(tree),BorderLayout.CENTER);
		
		
		
		// Create the buttons at the bottom.
		ButtonListener l = new ButtonListener();
		
		JPanel buttonPanel = new JPanel();
		
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(l);
		buttonPanel.add(cancelButton);

		downloadButton = new JButton("Download");
		downloadButton.setActionCommand("download");
		downloadButton.setEnabled(false);
		downloadButton.addActionListener(l);
		getRootPane().setDefaultButton(downloadButton);
		buttonPanel.add(downloadButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		
		setVisible(true);
	}

	private class ButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent ae) {
			if (ae.getActionCommand().equals("download")) {
				setVisible(false);
				GenomeNode selectedGenome = (GenomeNode)tree.getSelectionPath().getLastPathComponent();
				application.downloadGenome(selectedGenome.species,selectedGenome.assembly,selectedGenome.size);
				dispose();
			}
			else if (ae.getActionCommand().equals("cancel")) {
				setVisible(false);
				dispose();
			}
			
		}
	}
	
	private class TreeListener implements TreeSelectionListener {

		public void valueChanged(TreeSelectionEvent tse) {
			if (tree.getSelectionPath() != null && tree.getSelectionPath().getLastPathComponent() instanceof GenomeNode) {
				downloadButton.setEnabled(true);
			}
			else {
				downloadButton.setEnabled(false);
			}
		}
		
	}
	
	private class GenomeNode extends DefaultMutableTreeNode {
		
		public String species;
		public String assembly;
		public int size;
		
		public GenomeNode (String species, String assembly, int size) {
			super(assembly);
			this.species = species;
			this.assembly = assembly;
			this.size = size;
		}
				
	}
	
}
