/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;

public class SampleEditor extends JDialog implements ActionListener, ListSelectionListener {
	
	private ChipMonkApplication application;
	private DefaultListModel sampleModel = new DefaultListModel();
	private JList sampleList;
	private JButton deleteButton;
	private JButton renameButton;
	private boolean samplesDeleted = false;
	
	public SampleEditor (ChipMonkApplication application) {
		super(application,"Edit Samples...");
		setSize(300,400);
		setLocationRelativeTo(application);
		
		this.application = application;
		
		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.9;
		c.weighty=0.9;
		c.fill = GridBagConstraints.BOTH;
		
		JPanel samplePanel = new JPanel();
		samplePanel.setLayout(new GridBagLayout());
		GridBagConstraints gc = new GridBagConstraints();
		gc.gridx=0;
		gc.gridy=0;
		gc.weightx=0.5;
		gc.weighty=0;
		gc.fill = GridBagConstraints.HORIZONTAL;
		samplePanel.add(new JLabel("Samples",JLabel.CENTER),gc);
		
		gc.gridy++;
		gc.weighty=1;
		gc.fill = GridBagConstraints.BOTH;
		
		sampleList = new JList(sampleModel);
		sampleList.addListSelectionListener(this);
		samplePanel.add(new JScrollPane(sampleList),gc);

		gc.gridy++;
		gc.weighty=0;
		gc.fill = GridBagConstraints.HORIZONTAL;

		deleteButton = new JButton("Delete Sample");
		deleteButton.setActionCommand("delete_sample");
		deleteButton.addActionListener(this);
		deleteButton.setEnabled(false);
		samplePanel.add(deleteButton,gc);
		
		gc.gridy++;
		renameButton = new JButton("Rename Sample");
		renameButton.setActionCommand("rename_sample");
		renameButton.addActionListener(this);
		renameButton.setEnabled(false);
		samplePanel.add(renameButton,gc);
		
		getContentPane().add(samplePanel,c);

		c.gridx++;
		
		c.gridx=0;
		c.gridy++;
		c.weighty = 0.1;
		c.fill = GridBagConstraints.NONE;

		JPanel bottomPanel = new JPanel();
		
		JButton closeButton = new JButton("Close");
		closeButton.setActionCommand("close");
		closeButton.addActionListener(this);
		bottomPanel.add(closeButton);
		
		getContentPane().add(bottomPanel,c);
		
		// Fill the lists with the data we know about
		DataSet [] dataSets = application.dataCollection().getAllDataSets();
		for (int i=0;i<dataSets.length;i++) {
			sampleModel.addElement(dataSets[i]);
		}
				
		setVisible(true);
	}

	public void actionPerformed(ActionEvent ae) {
		String c = ae.getActionCommand();

		if (c.equals("rename_sample")) {
			
			DataSet s = (DataSet)sampleList.getSelectedValue();
			String sampleName=null;
			while (true) {
				sampleName = (String)JOptionPane.showInputDialog(this,"Enter sample name","Sample Name",JOptionPane.QUESTION_MESSAGE,null,null,s.name());
				if (sampleName == null) return; // They cancelled
				
				if (sampleName.length()>0) break;
			}
			s.setName(sampleName);
			sampleModel.setElementAt(s,sampleList.getSelectedIndex());
		}

		else if (c.equals("delete_sample")) {
			Object [] o = sampleList.getSelectedValues();
			for (int i=0;i<o.length;i++) {
				application.dataCollection().removeDataSet((DataSet)o[i]);
				application.removeFromDrawnDataStores((DataStore)o[i]);
				sampleModel.removeElement(o[i]);
				samplesDeleted = true;
			}
		}
		
		else if (c.equals("close")) {
			setVisible(false);
			if (samplesDeleted) {
				application.applyNormalisation();
			}
			dispose();
		}		
	}

	public void valueChanged(ListSelectionEvent ae) {
		if (sampleList.getSelectedValues().length > 0) {
			deleteButton.setEnabled(true);
		}
		else {
			deleteButton.setEnabled(false);
		}
		
		if (sampleList.getSelectedValues().length == 1) {
			renameButton.setEnabled(true);
		}
		else {
			renameButton.setEnabled(false);
		}
	}
}
