/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Displays.GenomeViewer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Chromosome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Genome;

public class GenomeViewer extends JPanel {

	private Chromosome [] chromosomes;
	private ChromosomeDisplay [] chromosomeDisplays;
	private ChipMonkApplication application;
	
	public GenomeViewer (Genome genome, ChipMonkApplication application) {
		chromosomes = genome.getAllChromosomes();
		chromosomeDisplays = new ChromosomeDisplay [chromosomes.length];
		this.application = application;

		setLayout(new BorderLayout());

		JLabel title = new JLabel ("Chromosomes in "+genome.species()+ " "+genome.assembly()+" assembly",JLabel.CENTER);
		add(title,BorderLayout.NORTH);
		
		
		JPanel chromosomePanel = new JPanel();
		chromosomePanel.setBackground(Color.WHITE);
		chromosomePanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.weightx=1;
		c.weighty=1;
		c.gridy=0; 
		c.fill = GridBagConstraints.BOTH;
		for (int i=0;i<chromosomes.length;i++) {
			c.gridy=i;
			c.gridx=0;
			c.weightx=0.1;
			chromosomePanel.add(new JLabel(chromosomes[i].name()),c);
			c.gridx=1;
			c.weightx=100;
			chromosomeDisplays[i] = new ChromosomeDisplay(genome,chromosomes[i],this);
			chromosomePanel.add(chromosomeDisplays[i],c);
		}
		
		chromosomePanel.addMouseListener(new MouseListener() {
			public void mouseExited(MouseEvent arg0) {
				setInfo(null);
			}		
			public void mouseEntered(MouseEvent arg0) {}		
			public void mouseReleased(MouseEvent arg0) {}		
			public void mousePressed(MouseEvent arg0) {}
			public void mouseClicked(MouseEvent arg0) {}
		});
		
		add(new JScrollPane(chromosomePanel),BorderLayout.CENTER);				
	}
	
	
	public ChipMonkApplication application () {
		return application;
	}
	
	public void setInfo (Chromosome c) {
		if (c == null) {
			application.setStatusText("ChIP Monk");
		}
		else {
			application.setStatusText("Chromosome "+c.name()+" "+c.length()+"bp");
		}
	}
	
	
	public void setView (Chromosome c, int start, int end) {
		for (int i=0;i<chromosomeDisplays.length;i++) {
			chromosomeDisplays[i].setView(c,start,end);
		}
		
	}
}
