/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Displays.MAPlot;

import java.awt.Color;
import java.awt.Image;

import javax.swing.JPanel;

import java.awt.Graphics;
import java.awt.image.BufferedImage;

import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataCollection;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;

public class MAPlotPanel extends JPanel {
	
	private static double log2 = Math.log(2);
	private double [] mValues;
	private double [] aValues;
	private double highestA;
	private double lowestA = 0;
	private double largestM = 0;
	private String name;
	
	public MAPlotPanel (DataCollection c, DataStore d) {
		setBackground(Color.WHITE);
		name = d.name();
		
		// We're going to scale this plot to the whole probe collection
		// even when we're only viewing a subset.  That way we can be
		// sure we can compare different plots.
		
		
		Probe [] probes = c.probeSet().getAllProbes();
		double highestM=0;
		double lowestM=0;
		highestA=0;
		lowestA=0;
		mValues = new double [probes.length];
		aValues = new double [probes.length];
		
		for (int i=0;i<probes.length;i++) {
			double signal = d.getSignalValueForProbe(probes[i]);
			double control = d.getControlValueForProbe(probes[i]);
			mValues[i] = Math.log(signal/control)/log2;
			aValues[i] = Math.log(Math.sqrt(signal*control))/log2;
			if (i==0) {
				highestM = mValues[0];
				lowestM = mValues[0];
				highestA = aValues[0];
				lowestA = aValues[0];
			}
			else {
				if (mValues[i]>highestM) highestM=mValues[i];
				if (mValues[i]<lowestM) lowestM=mValues[i];
				if (aValues[i]>highestA) highestA=aValues[i];
				if (aValues[i]<lowestA) lowestA=aValues[i];
			}
		}
		
		// Since M values center around 0 we need to pick the 
		// value with the highest magnitude to scale to
		if (highestM > (0-lowestM)) {
			largestM = highestM;
		}
		else {
			largestM = (0-lowestM);
		}
		
		// Now go through again using just the subset we actually want
		// to view
		probes = c.probeSet().getActiveProbes();
		mValues = new double [probes.length];
		aValues = new double [probes.length];
		
		for (int i=0;i<probes.length;i++) {
			double signal = d.getSignalValueForProbe(probes[i]);
			double control = d.getControlValueForProbe(probes[i]);
			mValues[i] = Math.log(signal/control)/log2;
			aValues[i] = Math.log(Math.sqrt(signal*control))/log2;
		}
		
	}

	public Image getCurrentView() {
		BufferedImage b = new BufferedImage(getWidth(),getHeight(),BufferedImage.TYPE_INT_RGB);
		Graphics g = b.getGraphics();			
		this.paint(g);
		return b;
	}
	
	public void paintComponent (Graphics g) {
		super.paintComponent(g);
		
		g.setColor(Color.RED);
		for (int i=0;i<mValues.length;i++) {
			int [] coords = getCoordinates(mValues[i],aValues[i]);
//			if (i%10000 == 0) {
//				System.out.println("M="+mValues[i]+" A="+aValues[i]+" x="+coords[0]+" y="+coords[1]);
//			}
			g.fillRect(coords[0]-1,coords[1]-1,2,2);
		}

		//Draw the axes and labels
		
		g.setColor(Color.LIGHT_GRAY);

		// X axis
		g.drawLine(20,getHeight()/2,getWidth()-5,getHeight()/2);

		// Box around plot
		g.drawRect(20,10,getWidth()-25,getHeight()-20);
		
		g.setColor(Color.BLACK);
		
		//Y-axis
		g.drawLine(20,10,20,getHeight()-10);
		
		// Y axis label
		g.drawString("M",24,20);
		
		// X axis label
		g.drawString("A",24,(getHeight()/2)-2);
		
		// Y axis origin
		g.drawString("0",10,(getHeight()/2)+3);
		
		// Filename at top
		g.drawString(name,2,10);
			
	}
	
	private int [] getCoordinates (double m, double a) {
		int [] r = new int [2];
		
		// The x value is offset by 20 at the left and 5 at the right
		r[0] = (int)(20+ (((double)getWidth()-25)  *  ((a-lowestA)/(highestA-lowestA))));
		
		// The y value is offset by 10 at top and bottom
		r[1] = (int) ((getHeight()-10) - (((double)getHeight()-20)* ((m+largestM)/(largestM*2))));
		
		return r;
	}
}
